// Title:  BiDiPro DLL module
// Desc:   Firmware HUB 49LF (p)
// Ver:    0.86b
// Found:  22-07-2003
// Last:   16-08-2003
// CPU:    386+
// Model:  Tiny
// Compil: Borland C++ 3.1 (DOS)
// Author: Grisha Lutsenko

// Based on SST and WINBOND datasheets

#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include "dlldata.h"
#include "errors.h"
#include "bidi.h"

//#define CMDLINE

#ifndef CMDLINE
 #define dlp ((DLLDATA *) NULL)        /* ⥫    */
#else
 extern DLLDATA far * dlp;
#endif //CMDLINE

#define bdport dlp->bdprt            /* 㪠⥫  hardware ࠩ */

#define DataPortIn()  bdport(PFORCE|PWR1|PBIN)
#define DataPortOut() bdport(PFORCE|PWR1)

struct  CHDEFS
{
   dword size;   // 
   byte  dpins;  //   dip 
   byte  rA2;    // ॣ  ⥭ //WE=1,CE=0,OE=0
   byte  rC1;    // use only for 256
   byte  rwB2;   // 殮
   byte  wC1h;   // :
   byte  wC1l;   //
   byte  wA2h;   // WE=1,CE=1,OE=1
   byte  wA2l;   // WE=0,CE=0,OE=1
};

CHDEFS chd[]={
//   size    pins  rA2   rC1  rwB2  wC1h  wC1l  wA2h  wA2l
  {0x040000L, 32, 0x00, 0x00, 0x81, 0x00, 0x00, 0x7C, 0x68},  //020
  {0x060000L, 32, 0x00, 0x00, 0x81, 0x00, 0x00, 0x7C, 0x68},  //030
  {0x080000L, 32, 0x00, 0x00, 0x81, 0x00, 0x00, 0x74, 0x60},  //040
  {0x100000L, 32, 0x00, 0x00, 0x81, 0x00, 0x00, 0x74, 0x60},  //080

};

// ७   CFG:
static struct CFGKEY cfgkeys[]={
{'D', "MaxWT", 50,50},   //max ⥫쭮 ppp        0
{'D', "ErTime",10000000,10000000},  //p 横 p          1
{'h', "Adr1",  0x05555,0x05555},  //                                2
{'h', "Adr2",  0x02AAA,0x02AAA},  //                                3
{'D', "Reset",1,1,1},      //reset on/off                           4
{0,    NULL,   0,0},     //End of table
};

#define cfg_MaxWT  cfgkeys[0].data
#define cfg_ETime  cfgkeys[1].data
#define cfg_Adr1   cfgkeys[2].data
#define cfg_Adr2   cfgkeys[3].data
#define cfg_Reset  cfgkeys[4].data

static char* copyright="49LF FirmWare Hub module (c) Grisha Lutsenko ["__DATE__"]";
static word chtype=0xFFFF;           //  ROM
static CHDEFS * pchd;

#define power_off breset
inline void set_adr(dword adr);

void breset (void)
{
   bdport (PRST);
}

void write_code (dword adr,byte dat)
{
     set_adr(adr);
     bdport (PWA2 | 0xA8);
//     dlp->mcsdelay(2);
     bdport (PWA2 | 0xB8);
//     dlp->mcsdelay(2);
     bdport (PWB1 | dat);
//     dlp->mcsdelay(2);
     bdport (PWA2 | 0xBC);
}

int power_on (byte mode)
{
   word vmask=pchd->rwB2;

   if (chtype == 0xFFFF) return ERC_CHIPUNSP;
   if (mode=='R')
    {
     DataPortIn();              // DD1:OIO
     bdport (PFORCE|PWR2);      // DD4:OOO
     bdport (PWA2 | 0xBC); 	//ce,oe,pe
//⠭ 殮
     dlp->voltage (dlp->Vcc,0,0);
     bdport (PWB2 | (vmask & 0xE0) ); //Vcc pins
    }
   else //'W' or other
    {
     bdport (PFORCE|PWR2);       // DD4:OOO
     bdport (PWA2 | 0xBC); 	 // ce,oe,pe
     DataPortOut();              // DD1:OOO
//⠭ 殮
     vmask&=0xE0; 		//Vpp disable
     dlp->voltage (dlp->Vcc,0,0);
     bdport (PWB2 | vmask ); 	//Vcc on
    }
   dlp->mcsdelay(100000);
   if(cfg_Reset==1)
    {
	bdport(PWA2 | 0);
       dlp->mcsdelay(100);
	bdport(PWA2 | 0xB4);
       dlp->mcsdelay(100);
	bdport(PWA2 | 0xBC);
       dlp->mcsdelay(5);
    }
   return 0;
}

int testinsert (void)
{
   if (dlp->testins==0) return 0; // ஢塞
   power_on('R');
   byte d1,d2;
// ⠭  0xFFFFF:
     bdport (PWA1 | 0xFF );
     bdport (PWC1 | pchd->rC1);
     bdport (PWA2 | pchd->rA2 | BITTER | 0x0F); //ter=1
     dlp->mcsdelay(10);             //প
     d1=bdport(PRB1);
     bdport (PWA2 | (pchd->rA2 & ~BITTER | 0x0F)); //ter=0
     dlp->mcsdelay(10);             //প
     d2=bdport(PRB1);
     bdport (PWA2 | pchd->rA2 | BITTER | 0x0F); //ter=1

   if (d1==d2)
	{	
     power_off();
     dlp->mcsdelay(3000);
     return 0; //ok
     }

   power_off();
   dlp->msgBox (dmfError|dmfOKButton,"\003Chip not connected!");
   return 1; //error

}

// ⠭ 
inline void set_adr(dword adr)
{
     dword adres;
     adres=adr;
     adr=adres&0x7FF;
     byte db;
//     dlp->mcsdelay(1);
// 1  :
     bdport (PWA1 | (adr&255));
// 2  :
     bdport (PWC1 | ((adr&0x700)>>8) );
//     dlp->mcsdelay(1);
// 3 ᨣ CE=0, OE=1, WE=1:
     bdport (PWA2 | 0xAC);
//     dlp->mcsdelay(1);


     adr=(adres&0x3FF800)>>11;
// 1  :
     bdport (PWA1 | (adr&255) );
// 2  :
     bdport (PWC1 | ((adr&0x700)>>8) );
//     dlp->mcsdelay(1);

}

inline byte read_one (dword adr)
{
     byte db;
     set_adr(adr);
// 3 ᨣ CE=1, OE=1, WE=1:
     bdport (PWA2 | 0xBC);
//     dlp->mcsdelay(1);

     bdport (PWA2 | 0x9C);
//     dlp->mcsdelay(1);

// DATA
     db=(byte) bdport(PRB1);
//     dlp->mcsdelay(1);
     bdport (PWA2 |0xBC);

     return db;
}

// 
void write_one (dword adr,byte dat)
{
write_code(cfg_Adr1,0xAA);
write_code(cfg_Adr2,0x55);
write_code(cfg_Adr1,0xA0);
write_code(adr,dat);
dlp->mcsdelay(cfg_MaxWT);
}

int read_all (void)
{
   dword adr;
   dword istep=pchd->size/64L;
   if (chtype == 0xFFFF) return ERC_CHIPUNSP;
  if(testinsert()==1) return 0;
   power_on('R');
   for (adr=0;adr<pchd->size;adr++)
    {
      dlp->setbytebuf (adr,read_one(adr));
      if (adr%istep == 0)
       if (dlp->iprogress(adr,2)) break;
    }
   power_off();
   return 0;
}

int write_all (void)
{
   dword adr;
   dword istep=pchd->size/64L;
   byte  tdr,db,t;

   if (chtype == 0xFFFF) return ERC_CHIPUNSP;
  if(testinsert()==1) return 0;
   power_on('W');

   for (adr=0; adr < pchd->size; adr++)
    {
      db=dlp->getbytebuf(adr);
      //⮢ 
      DataPortOut();
      disable();      // 뢠  ६  ࠭
      write_one(adr,db);
      enable();

      if(dlp->checkwr)
      {
	DataPortIn();
	tdr=read_one(adr);
	      if (tdr!=db) //compare error?
	       {
	         dlp->adr=adr;
	         (byte) dlp->edata=tdr;
	         power_off();
	         return ERC_WRITE;
	       }
      }
     if ((adr+1)%istep == 0)
     if (dlp->iprogress(adr,2)) break;

    }
   power_off();
   return 0;
}

int erase (void)
{
#define DISCRETE cfg_ETime/32

   dword i;
   byte tdr;

   if (chtype == 0xFFFF) return ERC_CHIPUNSP;
   if(testinsert()==1) return 0;
   dlp->iprogress(DISCRETE,0); //init
   power_on('W');
   disable();
   write_code(cfg_Adr1,0x0AA);
   write_code(cfg_Adr2,0x055);
   write_code(cfg_Adr1,0x080);
   write_code(cfg_Adr1,0x0AA);
   write_code(cfg_Adr2,0x055);
   write_code(cfg_Adr1,0x010);
   enable();
   DataPortIn();
   for (i=1;i<=cfg_ETime;i++)
    {
      dlp->mcsdelay(1);
      tdr=read_one(pchd->size-1);
      if (tdr & 0x80) break; //right data
      if (!(cfg_ETime % DISCRETE)) if (dlp->iprogress(i,2)) break;
    }
   power_off();
   dlp->iprogress(0,1); //close
   if (!tdr) return ERC_BLANK;
   return 0;
}

// Product Identification:
inline int read_id(void)
{
   dword id1,id2;
   dlp->edata=0;
   if (chtype == 0xFFFF) return ERC_CHIPUNSP;
  if(testinsert()==1) return 0;
   power_on('R');
     DataPortOut();

     disable();      // 뢠  ६ . ०. ⥭ ID
     write_code(cfg_Adr1,0x0AA);
     write_code(cfg_Adr2,0x055);
     write_code(cfg_Adr1,0x090);
     enable();
     dlp->mcsdelay(2);

   DataPortIn();               // DD1:OIO
   set_adr(0);
   bdport(PWA2 | 0x9C);
   dlp->mcsdelay(1);
   id1= (byte) bdport(PRB1);
   set_adr(1);
   bdport(PWA2 | 0x9C);
   dlp->mcsdelay(1);
// 
   id2= (byte) bdport(PRB1);
   dlp->edata = id1 | (id2<<16L);
   power_off();
   return 0;
}

// ⠢  ஬
int compare_all(void)
{
   dword adr;
   dword errors=0;
   dword istep=pchd->size/64L;
   if (chtype == 0xFFFF) return ERC_CHIPUNSP;
     if(testinsert()==1) return 0;
   power_on('R');
   for (adr=0;adr<pchd->size;adr++)
    {
      if (dlp->getbytebuf(adr) != read_one(adr) ) //ࠢ  ஬
       {
         dlp->setbmark(adr,1);  //⬥ ᮢ
         errors++;
       }
      if (adr%istep == 0)
       {
         if (dlp->iprogress(adr,2)) break;
       }
    }
   power_off();
   dlp->edata=errors;
   return 0;
}

int blank_chk(void)
{
   dword adr;
   dword istep=pchd->size/64L;
   if (chtype == 0xFFFF) return ERC_CHIPUNSP;
     if(testinsert()==1) return 0;
   power_on('R');
   for (adr=0;adr<pchd->size;adr++)
    {
      if (read_one(adr) != 0xFF ) //ࠢ  FF
       {
         dlp->adr=adr;
         power_off();
         return 1;
        }
       if (adr%istep == 0)
        if (dlp->iprogress(adr,2)) break;
    }
   power_off();
   return 0;
}


// ⠭ ⨯ ROM
word settype (void)
{
   for (word i=0;i<sizeof(chd)/sizeof(CHDEFS);i++)
    {
      if (chd[i].size == dlp->esize)
       return chtype=i;
    }
   return chtype=0xFFFF;
}

// main code
extern "C" int dmain(void)
{
   switch (dlp->funcode)
    {
      case DFN_LOAD:
// ⨢஢ 㯭 㭪樨:
       dlp->funmask[DFN_FREAD]=1;
       dlp->funmask[DFN_FWRITE]=1;
       dlp->funmask[DFN_READID]=1;
       dlp->funmask[DFN_ERASE]=1;
       dlp->funmask[DFN_FCOMPARE]=1;
       dlp->funmask[DFN_FBLANK]=1;
       dlp->funmask[DFN_TEST]=0;
       dlp->funmask[DFN_LOCK]=0;
       dlp->funmask[DFN_UNLOCK]=0;
       dlp->defstate=0xFF;
       dlp->copyright=copyright;
       dlp->version=DLLVERSION;
       dlp->cfgkeys=cfgkeys;
       return 0;
      case DFN_INIT:
       if (settype() == 0xFFFF) return ERC_CHIPUNSP;
       pchd=&chd[chtype];
       dlp->dippins=pchd->dpins;
       dlp->Vcc=55;
       return 0;

      case DFN_FREAD:
       return read_all();

      case DFN_FWRITE:
       return write_all();

      case DFN_FCOMPARE:
       return compare_all();

      case DFN_FBLANK:
       return blank_chk();

      case DFN_READID:
       return read_id();

      case DFN_ERASE:
       return erase();

      default:
       return ERC_FUNUNSP;
    }
}
